@@ -1,17 +1,24 @@
 from __future__ import annotations
 
 import os
+import sys
+from fractions import Fraction
+from os.path import splitext
+from subprocess import run
 from typing import Any
 
+import av
+from av import AudioResampler
+
 from auto_editor.ffwrapper import FFmpeg, FileInfo, initFileInfo
 from auto_editor.lib.contracts import is_int, is_str
-from auto_editor.make_layers import make_timeline
-from auto_editor.output import Ensure, mux_quality_media
+from auto_editor.make_layers import clipify, make_av, make_timeline
+from auto_editor.output import Ensure, parse_bitrate
 from auto_editor.render.audio import make_new_audio
 from auto_editor.render.subtitle import make_new_subtitles
 from auto_editor.render.video import render_av
 from auto_editor.timeline import v1, v3
-from auto_editor.utils.bar import Bar
+from auto_editor.utils.bar import initBar
 from auto_editor.utils.chunks import Chunk, Chunks
 from auto_editor.utils.cmdkw import ParserError, parse_with_palet, pAttr, pAttrs
 from auto_editor.utils.container import Container, container_constructor
@@ -25,7 +32,7 @@ def set_output(
     if src is None:
         root, ext = "out", ".mp4"
     else:
-        root, ext = os.path.splitext(str(src.path) if out is None else out)
+        root, ext = splitext(src.path if out is None else out)
         if ext == "":
             ext = src.path.suffix
 
@@ -45,8 +52,9 @@ def set_output(
 
     ext_map = {
         "premiere": ".xml",
-        "resolve": ".fcpxml",
+        "resolve-fcp7": ".xml",
         "final-cut-pro": ".fcpxml",
+        "resolve": ".fcpxml",
         "shotcut": ".mlt",
         "json": ".json",
         "audio": ".wav",
@@ -89,11 +97,19 @@ def set_audio_codec(
     codec: str, src: FileInfo | None, out_ext: str, ctr: Container, log: Log
 ) -> str:
     if codec == "auto":
-        codec = "aac" if (src is None or not src.audios) else src.audios[0].codec
+        if src is None or not src.audios:
+            codec = "aac"
+        else:
+            codec = src.audios[0].codec
+            ctx = av.Codec(codec)
+            if ctx.audio_formats is None:
+                codec = "aac"
         if codec not in ctr.acodecs and ctr.default_aud != "none":
-            return ctr.default_aud
+            codec = ctr.default_aud
         if codec == "mp3float":
-            return "mp3"
+            codec = "mp3"
+        if codec is None:
+            codec = "aac"
         return codec
 
     if codec == "copy":
@@ -103,9 +119,8 @@ def set_audio_codec(
             log.error("Input file does not have an audio stream to copy codec from.")
         codec = src.audios[0].codec
 
-    if codec != "unset":
-        if ctr.acodecs is None or codec not in ctr.acodecs:
-            log.error(codec_error.format(codec, out_ext))
+    if ctr.acodecs is None or codec not in ctr.acodecs:
+        log.error(codec_error.format(codec, out_ext))
 
     return codec
 
@@ -122,8 +137,11 @@ def parse_export(export: str, log: Log) -> dict[str, Any]:
     parsing: dict[str, pAttrs] = {
         "default": pAttrs("default"),
         "premiere": pAttrs("premiere", name_attr),
+        "resolve-fcp7": pAttrs("resolve-fcp7", name_attr),
+        "final-cut-pro": pAttrs(
+            "final-cut-pro", name_attr, pAttr("version", 11, is_int)
+        ),
         "resolve": pAttrs("resolve", name_attr),
-        "final-cut-pro": pAttrs("final-cut-pro", name_attr),
         "shotcut": pAttrs("shotcut"),
         "json": pAttrs("json", pAttr("api", 3, is_int)),
         "timeline": pAttrs("json", pAttr("api", 3, is_int)),
@@ -143,11 +161,11 @@ def parse_export(export: str, log: Log) -> dict[str, Any]:
 
 
 def edit_media(paths: list[str], ffmpeg: FFmpeg, args: Args, log: Log) -> None:
-    bar = Bar(args.progress)
+    bar = initBar(args.progress)
     tl = None
 
     if paths:
-        path_ext = os.path.splitext(paths[0])[1].lower()
+        path_ext = splitext(paths[0])[1].lower()
         if path_ext == ".xml":
             from auto_editor.formats.fcp7 import fcp7_read_xml
 
@@ -176,10 +194,11 @@ def edit_media(paths: list[str], ffmpeg: FFmpeg, args: Args, log: Log) -> None:
 
     del paths
 
-    output, export = set_output(args.output_file, args.export, src, log)
-    assert "export" in export
+    output, export_ops = set_output(args.output_file, args.export, src, log)
+    assert "export" in export_ops
+    export = export_ops["export"]
 
-    if export["export"] == "timeline":
+    if export == "timeline":
         log.quiet = True
 
     if not args.preview:
@@ -203,10 +222,10 @@ def edit_media(paths: list[str], ffmpeg: FFmpeg, args: Args, log: Log) -> None:
     if tl is None:
         tl = make_timeline(sources, args, samplerate, bar, log)
 
-    if export["export"] == "timeline":
+    if export == "timeline":
         from auto_editor.formats.json import make_json_timeline
 
-        make_json_timeline(export["api"], 0, tl, log)
+        make_json_timeline(export_ops["api"], 0, tl, log)
         return
 
     if args.preview:
@@ -215,31 +234,41 @@ def edit_media(paths: list[str], ffmpeg: FFmpeg, args: Args, log: Log) -> None:
         preview(tl, log)
         return
 
-    if export["export"] == "json":
+    if export == "json":
         from auto_editor.formats.json import make_json_timeline
 
-        make_json_timeline(export["api"], output, tl, log)
+        make_json_timeline(export_ops["api"], output, tl, log)
         return
 
-    if export["export"] == "premiere":
+    if export in ("premiere", "resolve-fcp7"):
         from auto_editor.formats.fcp7 import fcp7_write_xml
 
-        fcp7_write_xml(export["name"], output, tl, log)
+        is_resolve = export.startswith("resolve")
+        fcp7_write_xml(export_ops["name"], output, is_resolve, tl)
         return
 
-    if export["export"] in ("final-cut-pro", "resolve"):
+    if export == "final-cut-pro":
         from auto_editor.formats.fcp11 import fcp11_write_xml
 
-        fcp11_write_xml(export["name"], ffmpeg, output, export["export"], tl, log)
+        ver = export_ops["version"]
+        fcp11_write_xml(export_ops["name"], ver, output, False, tl, log)
         return
 
-    if export["export"] == "shotcut":
+    if export == "resolve":
+        from auto_editor.formats.fcp11 import fcp11_write_xml
+        from auto_editor.timeline import set_stream_to_0
+
+        set_stream_to_0(tl, log)
+        fcp11_write_xml(export_ops["name"], 10, output, True, tl, log)
+        return
+
+    if export == "shotcut":
         from auto_editor.formats.shotcut import shotcut_write_mlt
 
         shotcut_write_mlt(output, tl)
         return
 
-    out_ext = os.path.splitext(output)[1].replace(".", "")
+    out_ext = splitext(output)[1].replace(".", "")
 
     # Check if export options make sense.
     ctr = container_constructor(out_ext.lower())
@@ -253,66 +282,148 @@ def edit_media(paths: list[str], ffmpeg: FFmpeg, args: Args, log: Log) -> None:
     if args.keep_tracks_separate and ctr.max_audios == 1:
         log.warning(f"'{out_ext}' container doesn't support multiple audio tracks.")
 
-    def make_media(tl: v3, output: str) -> None:
+    def make_media(tl: v3, output_path: str) -> None:
         assert src is not None
 
-        visual_output = []
-        audio_output = []
-        sub_output = []
-        apply_later = False
+        output = av.open(output_path, "w")
 
-        ensure = Ensure(ffmpeg, bar, samplerate, log)
         if ctr.default_sub != "none" and not args.sn:
-            sub_output = make_new_subtitles(tl, ensure, log.temp)
+            sub_paths = make_new_subtitles(tl, log)
+        else:
+            sub_paths = []
 
         if ctr.default_aud != "none":
-            audio_output = make_new_audio(tl, ensure, args, ffmpeg, bar, log)
+            ensure = Ensure(bar, samplerate, log)
+            audio_paths = make_new_audio(tl, ctr, ensure, args, ffmpeg, bar, log)
+        else:
+            audio_paths = []
 
-        if ctr.default_vid != "none":
-            if tl.v:
-                out_path, apply_later = render_av(ffmpeg, tl, args, bar, ctr, log)
-                visual_output.append((True, out_path))
+        # Setup audio
+        if audio_paths:
+            try:
+                audio_encoder = av.Codec(args.audio_codec)
+            except av.FFmpegError as e:
+                log.error(e)
+            if audio_encoder.audio_formats is None:
+                log.error(f"{args.audio_codec}: No known audio formats avail.")
+            audio_format = audio_encoder.audio_formats[0]
+            resampler = AudioResampler(format=audio_format, layout="stereo", rate=tl.sr)
 
-            for v, vid in enumerate(src.videos, start=1):
-                if ctr.allow_image and vid.codec in ("png", "mjpeg", "webp"):
-                    out_path = os.path.join(log.temp, f"{v}.{vid.codec}")
-                    # fmt: off
-                    ffmpeg.run(["-i", f"{src.path}", "-map", "0:v", "-map", "-0:V",
-                        "-c", "copy", out_path])
-                    # fmt: on
-                    visual_output.append((False, out_path))
+        audio_streams: list[av.AudioStream] = []
+        audio_inputs = []
+        audio_gen_frames = []
+        for i, audio_path in enumerate(audio_paths):
+            audio_stream = output.add_stream(
+                args.audio_codec,
+                format=audio_format,
+                rate=tl.sr,
+                time_base=Fraction(1, tl.sr),
+            )
+            if not isinstance(audio_stream, av.AudioStream):
+                log.error(f"Not a known audio codec: {args.audio_codec}")
 
-        log.conwrite("Writing output file")
-        mux_quality_media(
-            ffmpeg,
-            visual_output,
-            audio_output,
-            sub_output,
-            apply_later,
-            ctr,
-            output,
-            tl.tb,
-            args,
-            src,
-            log,
-        )
+            if args.audio_bitrate != "auto":
+                audio_stream.bit_rate = parse_bitrate(args.audio_bitrate, log)
+                log.debug(f"audio bitrate: {audio_stream.bit_rate}")
+            else:
+                log.debug(f"[auto] audio bitrate: {audio_stream.bit_rate}")
+            if i < len(src.audios) and src.audios[i].lang is not None:
+                audio_stream.metadata["language"] = src.audios[i].lang  # type: ignore
 
-    if export["export"] == "clip-sequence":
+            audio_streams.append(audio_stream)
+            audio_input = av.open(audio_path)
+            audio_inputs.append(audio_input)
+            audio_gen_frames.append(audio_input.decode(audio=0))
+
+        # Setup subtitles
+        subtitle_streams = []
+        subtitle_inputs = []
+        sub_gen_frames = []
+
+        for i, sub_path in enumerate(sub_paths):
+            subtitle_input = av.open(sub_path)
+            subtitle_inputs.append(subtitle_input)
+            subtitle_stream = output.add_stream(
+                template=subtitle_input.streams.subtitles[0]
+            )
+            if i < len(src.subtitles) and src.subtitles[i].lang is not None:
+                subtitle_stream.metadata["language"] = src.subtitles[i].lang  # type: ignore
+
+            subtitle_streams.append(subtitle_stream)
+            sub_gen_frames.append(subtitle_input.demux(subtitles=0))
+
+        # Setup video
+        if ctr.default_vid != "none" and tl.v:
+            vframes = render_av(output, tl, args, bar, log)
+            output_stream = next(vframes)
+        else:
+            output_stream, vframes = None, iter([])
+
+        # Process frames
+        while True:
+            audio_frames = [next(frames, None) for frames in audio_gen_frames]
+            video_frame = next(vframes, None)
+            subtitle_frames = [next(packet, None) for packet in sub_gen_frames]
+
+            if (
+                all(frame is None for frame in audio_frames)
+                and video_frame is None
+                and all(packet is None for packet in subtitle_frames)
+            ):
+                break
+
+            for audio_stream, audio_frame in zip(audio_streams, audio_frames):
+                if audio_frame:
+                    for reframe in resampler.resample(audio_frame):
+                        output.mux(audio_stream.encode(reframe))
+
+            for subtitle_stream, packet in zip(subtitle_streams, subtitle_frames):
+                if not packet or packet.dts is None:
+                    continue
+                packet.stream = subtitle_stream
+                output.mux(packet)
+
+            if video_frame:
+                try:
+                    output.mux(output_stream.encode(video_frame))
+                except av.error.ExternalError:
+                    log.error(
+                        f"Generic error for encoder: {output_stream.name}\n"
+                        "Perhaps video quality settings are too low?"
+                    )
+                except av.FFmpegError as e:
+                    log.error(e)
+
+        # Flush streams
+        if output_stream is not None:
+            output.mux(output_stream.encode(None))
+        for audio_stream in audio_streams:
+            output.mux(audio_stream.encode(None))
+
+        # Close resources
+        for audio_input in audio_inputs:
+            audio_input.close()
+        for subtitle_input in subtitle_inputs:
+            subtitle_input.close()
+        output.close()
+
+    if export == "clip-sequence":
         if tl.v1 is None:
             log.error("Timeline too complex to use clip-sequence export")
 
-        from auto_editor.make_layers import clipify, make_av
-        from auto_editor.utils.func import append_filename
-
         def pad_chunk(chunk: Chunk, total: int) -> Chunks:
             start = [] if chunk[0] == 0 else [(0, chunk[0], 99999.0)]
             end = [] if chunk[1] == total else [(chunk[1], total, 99999.0)]
             return start + [chunk] + end
 
+        def append_filename(path: str, val: str) -> str:
+            root, ext = splitext(path)
+            return root + val + ext
+
         total_frames = tl.v1.chunks[-1][1] - 1
         clip_num = 0
         for chunk in tl.v1.chunks:
-            if chunk[2] == 99999:
+            if chunk[2] == 0 or chunk[2] >= 99999:
                 continue
 
             padded_chunks = pad_chunk(chunk, total_frames)
@@ -338,13 +449,23 @@ def edit_media(paths: list[str], ffmpeg: FFmpeg, args: Args, log: Log) -> None:
 
     log.stop_timer()
 
-    if not args.no_open and export["export"] in ("default", "audio", "clip-sequence"):
+    if not args.no_open and export in ("default", "audio"):
         if args.player is None:
-            from auto_editor.utils.func import open_with_system_default
-
-            open_with_system_default(output, log)
+            if sys.platform == "win32":
+                try:
+                    os.startfile(output)
+                except OSError:
+                    log.warning(f"Could not find application to open file: {output}")
+            else:
+                try:  # MacOS case
+                    run(["open", output])
+                except Exception:
+                    try:  # WSL2 case
+                        run(["cmd.exe", "/C", "start", output])
+                    except Exception:
+                        try:  # Linux case
+                            run(["xdg-open", output])
+                        except Exception:
+                            log.warning(f"Could not open output file: {output}")
         else:
-            import subprocess
-            from shlex import split
-
-            subprocess.run(split(args.player) + [output])
+            run(__import__("shlex").split(args.player) + [output])
