@@ -55,12 +55,9 @@ def codec_type(x: str) -> str:
         return "subtitle"
 
     try:
-        return Codec(x, "r").type
+        return Codec(x, "w").type
     except Exception:
-        try:
-            return Codec(x, "w").type
-        except Exception:
-            return ""
+        return ""
 
 
 def container_constructor(ext: str) -> Container:
@@ -80,6 +77,8 @@ def container_constructor(ext: str) -> Container:
     scodecs = set()
 
     for codec in codecs:
+        if ext == "wav" and codec == "aac":
+            continue
         kind = codec_type(codec)
         if kind == "video":
             vcodecs.add(codec)
